SystemOrganization addCategory: #'Magritte-XMLBinding'!

!MAElementDescription methodsFor: '*magritte-xmlbinding' stamp: 'jvds 10/10/2010 15:52'!
beXmlAttribute
	"Store objects as an attribute of the parent node."
	"example: <container attributeName='stringValue' />"
	self 
		beXmlExportable;
		xmlTarget: (MXXmlAttributeTarget new attributeName: self defaultXmlAttributeName)! !

!MAElementDescription methodsFor: '*magritte-xmlbinding' stamp: 'jvds 10/18/2010 21:22'!
beXmlChildAttribute
	"Store objects as an attribute of a child element."
	"example: <container><elementName attributeName='stringValue' /></container>"
	self 
		beXmlExportable;
		xmlTarget: (MXXmlNestedAttributeTarget new 
			elementName: self defaultXmlElementName;
			attributeName: self defaultXmlAttributeName)! !

!MAElementDescription methodsFor: '*magritte-xmlbinding' stamp: 'jvds 10/22/2010 11:01'!
beXmlElement
	"Store objects as the contents of a child element."
	"example: <container><child>....</child></container>"
	
	self 
		beXmlExportable;
		xmlTarget: (MXXmlNamedElementTarget new elementName: self defaultXmlElementName)! !

!MAElementDescription methodsFor: '*magritte-xmlbinding' stamp: 'jvds 10/22/2010 11:18'!
beXmlElementCData
	"Store objects as the contents of a child element. If the contents is Text than a CDATA
	section will be used."
	"example: <container><child><!![CDATA[ ... ]]></child></container>"
	
	self beXmlElement.
	self xmlTarget setUseCDATASection! !

!MAElementDescription methodsFor: '*magritte-xmlbinding' stamp: 'jvds 9/16/2010 16:09'!
defaultXmlAttributeName
	"Answer a default xml attribute name. Use the readSelector if present.
	Otherwise convert the label to a valid attribute name."
	
	^self defaultXmlElementName! !

!MAElementDescription methodsFor: '*magritte-xmlbinding' stamp: 'NickAger 2/10/2012 09:41'!
descriptionXmlAttributeName
	<magritteDescription>
	^ MAStringDescription new
		accessor: #xmlAttributeName;
		label: 'XML Attribute Name';
		priority: 1100;
		yourself! !

!MAElementDescription methodsFor: '*magritte-xmlbinding' stamp: 'jvds 10/10/2010 15:54'!
xmlAttributeName
	^self xmlTarget attributeName! !

!MAElementDescription methodsFor: '*magritte-xmlbinding' stamp: 'jvds 10/10/2010 16:05'!
xmlAttributeName: aString

	self beXmlExportable.
	self propertyAt: #xmlTarget ifAbsent: [
		self xmlTarget: (MXXmlAttributeTarget new attributeName: aString).
		^self ].
	self xmlTarget attributeName: aString! !

!MARelationDescription methodsFor: '*magritte-xmlbinding' stamp: 'NorbertHartl 11/13/2010 17:30'!
beXmlInlineElement
	"Store related objects directly in the container node."
	
	self 
		beXmlExportable;
		xmlTarget: (MXXmlElementTarget new) beInlined  ! !

MAVisitor subclass: #MXXmlBinding
	instanceVariableNames: 'object node'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-XMLBinding'!

!MXXmlBinding commentStamp: 'jvds 10/18/2010 21:33' prior: 0!
MXXmlBinding is an abstract superclass for reading and writing xml.

Instance Variables:
	object	<Object>
	node	<XMLElement | nil>!

!MXXmlBinding methodsFor: 'visiting-description' stamp: 'NickAger 1/31/2012 12:31'!
descriptionFor: aContainerObject
	"Answer the description to use for ==aContainerObject==. Subclasses may override this to work with
	alternative descriptions."

	^ aContainerObject magritteDescription! !

!MXXmlBinding methodsFor: 'accessing' stamp: 'jvds 10/7/2010 17:15'!
node
	^ node! !

!MXXmlBinding methodsFor: 'accessing' stamp: 'jvds 10/7/2010 17:15'!
node: anObject
	node := anObject! !

!MXXmlBinding methodsFor: 'accessing' stamp: 'jvds 10/7/2010 17:15'!
object
	^ object! !

!MXXmlBinding methodsFor: 'accessing' stamp: 'jvds 10/7/2010 17:15'!
object: anObject
	object := anObject! !

MXXmlBinding subclass: #MXXmlReader
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-XMLBinding'!

!MXXmlReader commentStamp: 'jvds 10/18/2010 21:34' prior: 0!
MXXmlReader reads data from XMLElements and creates objects using the Magritte descriptions.!

!MXXmlReader class methodsFor: 'instance creation' stamp: 'jvds 10/6/2010 21:39'!
read: aNode description: aDescription

	^self new read: aNode description: aDescription! !

!MXXmlReader methodsFor: 'public' stamp: 'jvds 10/6/2010 21:49'!
read: aNode description: aDescription

	self 
		node: aNode;
		visit: aDescription.
	^self object! !

!MXXmlReader methodsFor: 'private' stamp: 'jvds 10/6/2010 22:09'!
readText: aDescription

	^aDescription xmlTarget getTextFrom: self node! !

!MXXmlReader methodsFor: 'visiting-description' stamp: 'NorbertHartl 10/12/2011 15:10'!
visitContainer: aDescription
	"Create a new instance using the current node and ==aDescription== and read all the children.
	Note that we cannot use ==aDescription== for the children because the object might be an instance
	of a subclass and this subclass may have another description."
	
	| trueDescription |
	
	self object: (aDescription xmlTarget getInstanceFrom: self node description: aDescription).
	
	trueDescription := aDescription shouldBePreserved 
		ifTrue: [ aDescription ] 
		ifFalse: [ self descriptionFor: self object] .
	(((trueDescription = aDescription)
		ifTrue: [ aDescription ]
		ifFalse: [ trueDescription ])
			children select: [ :each | each isXmlExportable ]) do: [ :each | | value |
				value := each xmlReader read: self node description: each.
				self object write: value using: each ]! !

!MXXmlReader methodsFor: 'visiting-description' stamp: 'jvds 10/7/2010 21:44'!
visitElementDescription: aDescription

	| text |
	
	text := self readText: aDescription.
	self object: (text isNil
		ifTrue: [ nil ]
		ifFalse: [ aDescription fromString: text ])
	! !

!MXXmlReader methodsFor: 'visiting-description' stamp: 'NorbertHartl 11/02/2011 15:07'!
visitSingleOptionDescription: aDescription
	"If the option value is a class than just output the class name instead of the label"
	
	aDescription reference kind = Class
		ifTrue: [
			| text |
			text := self readText: aDescription.
			self object: (text isNil 
				ifTrue: [ nil ] 
				ifFalse: [ Smalltalk at: text asSymbol ])]
		ifFalse: [ 
			super visitSingleOptionDescription: aDescription ]! !

!MXXmlReader methodsFor: 'visiting-description' stamp: 'NorbertHartl 1/25/2011 15:51'!
visitToManyRelationDescription: aDescription

	| containerNode |
	
	containerNode := aDescription xmlTarget getNodeFrom: self node.
	containerNode isNil
		ifTrue: [ ^self ].

	self object: (OrderedCollection  streamContents: [ :stream |
		containerNode elementsDo: [ :element |
			| value |
			value := aDescription reference xmlReader read: element description: aDescription reference.
			value notNil
				ifTrue: [ stream nextPut: value ]]])
! !

!MXXmlReader methodsFor: 'visiting-description' stamp: 'NorbertHartl 11/13/2010 17:06'!
visitToOneRelationDescription: aDescription

	| containerNode |
	
	containerNode := aDescription xmlTarget getNodeFrom: self node.
	containerNode notNil
		ifTrue: [ | value |
			(aDescription reference notNil and: [aDescription reference xmlTarget isKindOf: MXXmlNamedElementTarget]) 
				ifTrue: [ | elem |
					elem := (aDescription reference xmlTarget getNodeFrom: containerNode ).
					value := aDescription reference xmlReader read: elem description: aDescription reference ]
				ifFalse: [ 
					containerNode elementsDo: [ :elem |
						value := aDescription reference xmlReader read: elem description: aDescription reference ]].
			value notNil
				ifTrue: [
					self object: value.
					^ self ]].
	self object: nil! !

MXXmlBinding subclass: #MXXmlWriter
	instanceVariableNames: 'containerNode'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-XMLBinding'!

!MXXmlWriter commentStamp: 'jvds 10/18/2010 21:34' prior: 0!
MXXmlWriter writes data to XMLElements using an object with its Magritte description.!

!MXXmlWriter class methodsFor: 'instance creation' stamp: 'jvds 10/3/2010 12:18'!
write: anObject description: aDescription to: aNode

	^self new write: anObject description: aDescription to: aNode
! !

!MXXmlWriter methodsFor: 'accessing' stamp: 'jvds 10/5/2010 22:10'!
containerNode
	^ containerNode! !

!MXXmlWriter methodsFor: 'accessing' stamp: 'jvds 10/5/2010 22:10'!
containerNode: anObject
	containerNode := anObject! !

!MXXmlWriter methodsFor: 'visiting-description' stamp: 'jvds 10/10/2010 21:50'!
visitContainer: aDescription
	"Write out all the elements of object that are xml exportable
	and are notNil."
	
	self node: (aDescription xmlTarget addNodeForInstance: self object to: self containerNode).
	
	(aDescription children select: [ :each | each isXmlExportable ]) do: [ :each |
		each xmlWriter 
			write: self object description: each to: self node ]! !

!MXXmlWriter methodsFor: 'visiting-description' stamp: 'NorbertHartl 11/22/2010 21:16'!
visitElementDescription: aDescription

	| value text |

	value := self object readUsing: aDescription.
	text := value ifNotNilDo: [ :v | aDescription toString: v ].
	self writeText: text description: aDescription! !

!MXXmlWriter methodsFor: 'visiting-description' stamp: 'NorbertHartl 11/30/2010 12:14'!
visitSingleOptionDescription: aDescription
	"If the option value is a class than just output the class name instead of the label"
	
	aDescription reference kind = Class
		ifTrue: [
			self 
				writeText: ((self object readUsing: aDescription) ifNotNilDo: [ :c | c name greaseString ])
				description: aDescription ]
		ifFalse: [ 
			super visitSingleOptionDescription: aDescription ]! !

!MXXmlWriter methodsFor: 'visiting-description' stamp: 'NorbertHartl 7/7/2011 14:42'!
visitToManyRelationDescription: aDescription

	| collection referenceDescription |
	
	collection := self object readUsing: aDescription.
	collection isNil
		ifFalse: [
			self node: (aDescription xmlTarget addNodeTo: self containerNode).
			collection do: [ :each |
				referenceDescription := aDescription isDescribedByReference 
					ifTrue: [ aDescription reference ] 
					ifFalse: [ self descriptionFor: each ].
				referenceDescription xmlWriter
					write: each description: referenceDescription to: self node ]].! !

!MXXmlWriter methodsFor: 'visiting-description' stamp: 'NorbertHartl 2/17/2011 19:21'!
visitToOneRelationDescription: aDescription

	| value referenceDescription |
	value := self object readUsing: aDescription.
	value isNil
		ifTrue: [
			self node: (aDescription xmlTarget addUndefinedTo: self containerNode)]
		ifFalse: [
			self node: (aDescription xmlTarget addNodeTo: self containerNode).
			referenceDescription := aDescription reference ifNil: [ self descriptionFor: value].
			referenceDescription xmlWriter
				write: value description: referenceDescription to: self node ]! !

!MXXmlWriter methodsFor: 'public' stamp: 'jvds 10/4/2010 21:31'!
write: anObject description: aDescription to: aNode

	self 
		containerNode: aNode;
		object: anObject;
		visit: aDescription.
	^self node! !

!MXXmlWriter methodsFor: 'private' stamp: 'jvds 10/18/2010 21:05'!
writeText: aString description: aDescription

	self node: (
		aString isNil
			ifTrue: [ aDescription xmlTarget addUndefinedTo: self containerNode ]
			ifFalse: [ aDescription xmlTarget addText: aString to: self containerNode ])! !

!MAContainer methodsFor: '*magritte-xmlbinding' stamp: 'NorbertHartl 10/12/2011 15:09'!
bePreserved
	self propertyAt: #preserve put: true ! !

!MAContainer methodsFor: '*magritte-xmlbinding' stamp: 'jvds 10/5/2010 23:28'!
defaultXmlTarget
	"Answer a default xml element name. Use the readSelector if present.
	Otherwise convert the label to a valid element name."

	^MXXmlClassNameElementTarget new! !

!MAContainer methodsFor: '*magritte-xmlbinding' stamp: 'NorbertHartl 10/12/2011 15:09'!
shouldBePreserved
	^ self propertyAt: #preserve ifAbsent: [ false ] ! !

!MAContainer methodsFor: '*magritte-xmlbinding' stamp: 'jvds 10/11/2010 18:26'!
xmlElementName: anElementName classAttribute: anAttributeName

	self beXmlExportable.
	self xmlTarget: (MXXmlClassNameAttributeTarget new
		elementName: anElementName;
		classAttributeName: anAttributeName)! !

Object subclass: #MXObject
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-XMLBinding'!

!MXObject commentStamp: 'jvds 10/18/2010 21:30' prior: 0!
MXObject can be used as a superclass for objects that need to be marshalled to xml.!

!MXObject class methodsFor: 'accessing' stamp: 'NickAger 2/1/2012 16:10'!
forElementNamed: aString
	self forElementNamed: aString notFound: [ self error: 'unknown element name: ', aString ]
	! !

!MXObject class methodsFor: 'accessing' stamp: 'NickAger 2/1/2012 16:10'!
forElementNamed: aString notFound: aNotFoundBlock
	self withAllConcreteClassesDo: [ :each |
		| element |
		element := each new.
		(element magritteDescription xmlElementName = aString)
			ifTrue: [ ^ element ] ].
		
	^ aNotFoundBlock value! !

!MXObject class methodsFor: 'instance creation' stamp: 'NickAger 1/31/2012 12:29'!
fromXml: aString
	^ self new magritteDescription fromXml: aString! !

!MXObject class methodsFor: 'instance creation' stamp: 'NickAger 1/31/2012 12:28'!
fromXmlNode: aString
	^ self new magritteDescription fromXmlNode: aString! !

!MXObject class methodsFor: 'testing' stamp: 'NorbertHartl 2/15/2010 11:27'!
isAbstract
	^ true! !

!MXObject class methodsFor: 'reflection' stamp: 'NorbertHartl 2/15/2010 11:27'!
withAllConcreteClasses
	^Array streamContents: [ :stream |
		self withAllConcreteClassesDo: [ :each |
			stream nextPut: each ] ]! !

!MXObject class methodsFor: 'reflection' stamp: 'NorbertHartl 2/15/2010 11:27'!
withAllConcreteClassesDo: aBlock
	self withAllSubclassesDo: [ :each |
		each isAbstract ifFalse: [
			aBlock value: each ] ]! !

!MXObject class methodsFor: 'accessing' stamp: 'NorbertHartl 02/18/2010 14:42'!
xmlElementName
	self subclassResponsibility! !

!MXObject methodsFor: 'accessing-description' stamp: 'NickAger 1/31/2012 12:01'!
descriptionContainer
	<magritteContainer>
	^ super descriptionContainer
		xmlElementName: self class xmlElementName;
		yourself! !

!MXObject methodsFor: 'printing' stamp: 'NickAger 2/2/2012 18:22'!
printXMLOn: aStream
	self greaseDeprecatedApi: 'MXObject>>printXMLOn:' details: 'use #writeXMLOn: instead'.
	self writeXMLOn: aStream! !

!MXObject methodsFor: 'converting' stamp: 'NorbertHartl 12/21/2010 14:40'!
toXml
	^ self toXml: XMLDocument new! !

!MXObject methodsFor: 'converting' stamp: 'NickAger 1/31/2012 12:02'!
toXml: aParentNode
	^ self toXml: aParentNode using: self magritteDescription! !

!MXObject methodsFor: 'converting' stamp: 'NickAger 2/3/2012 16:45'!
toXml: aParentNode using: aDescription
	^ aDescription toXmlNode: self parent: aParentNode! !

!MXObject methodsFor: 'converting' stamp: 'NorbertHartl 12/21/2010 15:14'!
toXmlDocument
	^ self toXml document! !

!MXObject methodsFor: 'printing' stamp: 'NickAger 1/31/2012 15:13'!
writeXMLOn: aStream
	self toXml writeXMLOn: (XMLWriter on: aStream)! !

Object subclass: #MXPluggableXmlAccessor
	instanceVariableNames: 'readBlock writeBlock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-XMLBinding'!

!MXPluggableXmlAccessor commentStamp: 'jvds 10/18/2010 21:32' prior: 0!
MXPluggableXmlAccessor can be used as an alternative to the MXXmlWriter and MXXmlReader classes for customized mapping of an object to xml.

Instance Variables:
	readBlock	<Block>
	writeBlock	<Block>!

!MXPluggableXmlAccessor methodsFor: 'model' stamp: 'jvds 10/10/2010 15:26'!
read: aNode description: aDescription
	^self readBlock value: aNode! !

!MXPluggableXmlAccessor methodsFor: 'accessing' stamp: 'jvds 10/10/2010 15:22'!
readBlock
	^ readBlock! !

!MXPluggableXmlAccessor methodsFor: 'accessing' stamp: 'jvds 10/10/2010 15:22'!
readBlock: anObject
	readBlock := anObject! !

!MXPluggableXmlAccessor methodsFor: 'model' stamp: 'jvds 10/10/2010 15:42'!
write: anObject description: aDescription to: aNode
	self writeBlock 
		value: aNode value: (anObject readUsing: aDescription)! !

!MXPluggableXmlAccessor methodsFor: 'accessing' stamp: 'jvds 10/10/2010 15:22'!
writeBlock
	^ writeBlock! !

!MXPluggableXmlAccessor methodsFor: 'accessing' stamp: 'jvds 10/10/2010 15:22'!
writeBlock: anObject
	writeBlock := anObject! !

Object subclass: #MXXmlTarget
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-XMLBinding'!

!MXXmlTarget commentStamp: 'jvds 10/18/2010 21:37' prior: 0!
MXXmlTarget is the abstract root of a utitity class hierarchy used by MXXmlReader and MXXmlWriter to access the xml node tree structure.!

MXXmlTarget subclass: #MXXmlAttributeTarget
	instanceVariableNames: 'attributeName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-XMLBinding'!

!MXXmlAttributeTarget commentStamp: 'jvds 10/18/2010 21:39' prior: 0!
MXXmlAttributeTarget stores and reads text from xml attributes. For example:

	<container attributeName="value"></container>

Instance Variables:
	attributeName	<String>!

!MXXmlAttributeTarget methodsFor: 'writing' stamp: 'jvds 10/10/2010 21:55'!
addText: aString to: aContainerNode
	"Store ==aString== as an attribute in the element ==aContainerNode==."
	

	^aContainerNode
		attributeAt: self attributeName put: aString;
		yourself! !

!MXXmlAttributeTarget methodsFor: 'accessing' stamp: 'jvds 10/4/2010 22:33'!
attributeName
	^ attributeName! !

!MXXmlAttributeTarget methodsFor: 'accessing' stamp: 'jvds 10/4/2010 22:34'!
attributeName: aString
	attributeName := aString! !

!MXXmlAttributeTarget methodsFor: 'reading' stamp: 'jvds 10/6/2010 22:55'!
getTextFrom: anElement

	^anElement attributeAt: self attributeName ifAbsent: [ nil ]! !

MXXmlAttributeTarget subclass: #MXXmlNestedAttributeTarget
	instanceVariableNames: 'elementName useExisting'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-XMLBinding'!

!MXXmlNestedAttributeTarget commentStamp: 'jvds 10/18/2010 21:43' prior: 0!
MXXmlNestedAttributeTarget stores and reads text from xml attributes. For example:

	<container><elementName attributeName="value" /></container>

Instance Variables:
	elementName	<String>
	useExisting	<Boolean> - If the container elements already contains an element with the correct name and useExisting is true, 
		the attribute will be added to the existing element. Otherwise a new element is created.!

!MXXmlNestedAttributeTarget methodsFor: 'writing' stamp: 'NorbertHartl 12/21/2010 14:32'!
addText: aString to: aContainerNode
	"Store ==aString== as an attribute in the element ==aContainerNode==."
	
	| childElement |

	childElement := self useExisting 
		ifTrue: [ aContainerNode elementAt: self elementName ]
		ifFalse: [ nil ].
	childElement isNil
		ifTrue: [
			childElement := XMLElement named: self elementName.
			aContainerNode 
				addNode: childElement ].
	^childElement
		attributeAt: self attributeName put: aString;
		yourself
! !

!MXXmlNestedAttributeTarget methodsFor: 'accessing' stamp: 'jvds 10/10/2010 22:06'!
elementName
	^ elementName! !

!MXXmlNestedAttributeTarget methodsFor: 'accessing' stamp: 'jvds 10/10/2010 22:06'!
elementName: anObject
	elementName := anObject! !

!MXXmlNestedAttributeTarget methodsFor: 'reading' stamp: 'jvds 10/10/2010 22:08'!
getTextFrom: anElement

	| childElement |
	childElement := anElement elementAt: self elementName.
	^childElement isNil
		ifTrue: [ nil ]
		ifFalse: [ childElement attributeAt: self attributeName ]! !

!MXXmlNestedAttributeTarget methodsFor: 'initialize-release' stamp: 'jvds 10/10/2010 22:12'!
initialize

	super initialize.
	useExisting := false! !

!MXXmlNestedAttributeTarget methodsFor: 'accessing' stamp: 'jvds 10/10/2010 22:38'!
useExisting
	^useExisting! !

!MXXmlNestedAttributeTarget methodsFor: 'accessing' stamp: 'jvds 10/10/2010 22:38'!
useExisting: aBoolean
	useExisting := aBoolean! !

MXXmlTarget subclass: #MXXmlElementTarget
	instanceVariableNames: 'inlined'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-XMLBinding'!

!MXXmlElementTarget commentStamp: 'jvds 10/18/2010 21:45' prior: 0!
MXXmlElementTarget is an abstract class with support for storing and reading text as the contents of an element. And for adding 
and reading child elements.!

MXXmlElementTarget subclass: #MXXmlClassNameElementTarget
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-XMLBinding'!

!MXXmlClassNameElementTarget commentStamp: 'jvds 10/18/2010 21:48' prior: 0!
MXXmlClassNameElementTarget uses the class name of the object as the element name. For example:

	<MXTestPerson>........</MXTestPerson>!

!MXXmlClassNameElementTarget methodsFor: 'internal' stamp: 'jvds 10/22/2010 10:02'!
elementNameFor: anObject

	^anObject class name greaseString! !

!MXXmlClassNameElementTarget methodsFor: 'reading' stamp: 'NorbertHartl 11/02/2011 15:07'!
getInstanceFrom: anElement description: aDescription

	| class |
	class := Smalltalk 
		at: anElement name asSymbol
		ifAbsent: [ ^nil ].
	^class new! !

!MXXmlElementTarget methodsFor: 'writing' stamp: 'NorbertHartl 12/21/2010 15:12'!
addNodeForInstance: anObject to: aContainerNode
	"Create and answer a new element node that represents ==anObject==."
	
	self isInlined ifTrue: [ ^ aContainerNode ].
	^ self addNodeNamed: (self elementNameFor: anObject) toNode: aContainerNode
! !

!MXXmlElementTarget methodsFor: 'writing' stamp: 'NorbertHartl 12/21/2010 15:12'!
addNodeNamed: aString toNode: aContainerNode
	"Create and answer a new element node. Add it to ==aContainerNode=="
	
	| node |
	node := XMLElement named: aString.
	aContainerNode addNode: node.
	^ node! !

!MXXmlElementTarget methodsFor: 'writing' stamp: 'NorbertHartl 12/21/2010 15:12'!
addNodeTo: aContainerNode
	"Create and answer a new element node. Add it to ==aContainerNode=="
	
	self isInlined ifTrue: [ ^ aContainerNode ].
	^ self addNodeNamed: self elementName toNode: aContainerNode 
! !

!MXXmlElementTarget methodsFor: 'writing' stamp: 'jvds 10/22/2010 10:59'!
addText: aString to: aContainerNode
	"Create a new element with ==aString== as the contents. Add the new element to ==aContainerNode== if present.
	Answer the newly created element."

	| node |
	
	node := self addNodeTo: aContainerNode.
	
	self useCDATASection 
		ifTrue: [ node addNode: (XMLCData string: aString) ]
		ifFalse: [ node addContent: aString ].
	^node! !

!MXXmlElementTarget methodsFor: 'accessing' stamp: 'NorbertHartl 11/13/2010 17:26'!
beInlined
	inlined := true! !

!MXXmlElementTarget methodsFor: 'accessing' stamp: 'jvds 10/5/2010 23:20'!
elementName
	^self subclassResponsibility! !

!MXXmlElementTarget methodsFor: 'internal' stamp: 'jvds 10/10/2010 22:00'!
elementNameFor: anObject

	^self elementName! !

!MXXmlElementTarget methodsFor: 'reading' stamp: 'NickAger 2/1/2012 16:12'!
getInstanceFrom: anElement description: aDescription
	"Answer an instance of the correct class for ==anElement==."

	aDescription kind = Object
		ifTrue: [ 
			"assume a subclass of MXObject"
			^ MXObject forElementNamed: anElement name notFound: [ nil ] ]
		ifFalse: [ 
			^ aDescription kind withAllSubclassesDo: [ :class |
				| element |
				element := class new.
				element magritteDescription xmlElementName = anElement name
					ifTrue: [ ^ element] ] ].
	^ nil! !

!MXXmlElementTarget methodsFor: 'reading' stamp: 'NorbertHartl 11/13/2010 17:28'!
getNodeFrom: aContainerNode
	self isInlined ifTrue: [
		^ aContainerNode ].
	^aContainerNode elementAt: self elementName! !

!MXXmlElementTarget methodsFor: 'reading' stamp: 'NorbertHartl 11/29/2010 21:51'!
getTextFrom: aContainerNode

	^(self getNodeFrom: aContainerNode)
		ifNotNilDo: [ :elem | elem contentString ]! !

!MXXmlElementTarget methodsFor: 'initialize-release' stamp: 'NorbertHartl 11/13/2010 18:47'!
initialize
	super initialize.
	inlined := false! !

!MXXmlElementTarget methodsFor: 'testing' stamp: 'NorbertHartl 11/13/2010 17:27'!
isInlined
	^ inlined ifNil: [
		inlined := false]! !

!MXXmlElementTarget methodsFor: 'testing' stamp: 'jvds 10/22/2010 10:55'!
useCDATASection

	^false! !

MXXmlElementTarget subclass: #MXXmlNamedElementTarget
	instanceVariableNames: 'elementName useCDATASection'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-XMLBinding'!

!MXXmlNamedElementTarget commentStamp: 'jvds 10/18/2010 21:49' prior: 0!
MXXmlNamedElementTarget stores text and child elements in an element with a fixed name.

	<elementName>........</elementName>

Instance Variables:
	elementName	<String>!

MXXmlNamedElementTarget subclass: #MXXmlClassNameAttributeTarget
	instanceVariableNames: 'classAttributeName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magritte-XMLBinding'!

!MXXmlClassNameAttributeTarget commentStamp: 'jvds 10/18/2010 21:50' prior: 0!
MXXmlNamedElementTarget stores text and child elements in an element with a fixed name. The class name of the stored object is saved in an attribute.

	<elementName type="MXTestPerson">........</elementName>

Instance Variables:
	classAttributeName <String>!

!MXXmlClassNameAttributeTarget methodsFor: 'writing' stamp: 'jvds 10/10/2010 22:52'!
addNodeForInstance: anObject to: aContainerNode
	"Create and answer a new element node that represents ==anObject==."

	^(super addNodeForInstance: anObject to: aContainerNode)
		attributeAt: self classAttributeName put: anObject class name;
		yourself! !

!MXXmlClassNameAttributeTarget methodsFor: 'accessing' stamp: 'jvds 10/10/2010 22:48'!
classAttributeName
	^ classAttributeName! !

!MXXmlClassNameAttributeTarget methodsFor: 'accessing' stamp: 'jvds 10/10/2010 22:48'!
classAttributeName: anObject
	classAttributeName := anObject! !

!MXXmlClassNameAttributeTarget methodsFor: 'reading' stamp: 'NorbertHartl 11/02/2011 15:15'!
getInstanceFrom: anElement description: aDescription

	| name class |
	name := anElement attributeAt: self classAttributeName.
	class := Smalltalk 
		at: name asSymbol
		ifAbsent: [ ^nil ].
	^class new! !

!MXXmlClassNameAttributeTarget methodsFor: 'initialize-release' stamp: 'NorbertHartl 11/15/2010 00:14'!
initialize

	super initialize.
	classAttributeName := 'type'
! !

!MXXmlNamedElementTarget methodsFor: 'accessing' stamp: 'jvds 10/5/2010 23:21'!
elementName

	^elementName ! !

!MXXmlNamedElementTarget methodsFor: 'accessing' stamp: 'jvds 10/5/2010 23:21'!
elementName: aString

	elementName := aString! !

!MXXmlNamedElementTarget methodsFor: 'initialize-release' stamp: 'jvds 10/22/2010 10:52'!
initialize

	super initialize.
	useCDATASection := false! !

!MXXmlNamedElementTarget methodsFor: 'accessing' stamp: 'jvds 10/22/2010 10:52'!
setUseCDATASection

	useCDATASection := true! !

!MXXmlNamedElementTarget methodsFor: 'accessing' stamp: 'jvds 10/22/2010 10:55'!
useCDATASection

	^useCDATASection! !

!MXXmlTarget methodsFor: 'writing' stamp: 'jvds 10/4/2010 22:05'!
addText: aString to: aContainerNode

	^aContainerNode
		addContent: aString;
		yourself! !

!MXXmlTarget methodsFor: 'writing' stamp: 'jvds 10/14/2010 14:53'!
addUndefinedTo: aContainerNode
	"Add a undefined (nil) value to aContainerNode. The default implementation does nothing"

	^aContainerNode! !

!MXXmlTarget methodsFor: 'reading' stamp: 'jvds 10/10/2010 21:58'!
getTextFrom: anElement

	^anElement contentString! !

!MADescription class methodsFor: '*magritte-xmlbinding' stamp: 'jvds 10/18/2010 21:27'!
defaultXmlReader
	^MXXmlReader ! !

!MADescription class methodsFor: '*magritte-xmlbinding' stamp: 'jvds 10/18/2010 21:27'!
defaultXmlWriter
	^MXXmlWriter ! !

!MADescription methodsFor: '*magritte-xmlbinding' stamp: 'NorbertHartl 2/19/2010 17:53'!
beXmlExportable
	self propertyAt: #xmlExportable put: true! !

!MADescription methodsFor: '*magritte-xmlbinding' stamp: 'jvds 9/16/2010 08:54'!
defaultXmlElementName
	"Answer a default xml element name. Use the readSelector if present.
	Otherwise convert the label to a valid element name."

	| name |
	
	(self accessor respondsTo: #readSelector)
		ifTrue: [ ^self accessor readSelector greaseString ].
		
	name := self label select: [ :char | char isAlphaNumeric ].
	name size > 0
		ifTrue: [ name at: 1 put: name first asLowercase ].
	^name! !

!MADescription methodsFor: '*magritte-xmlbinding' stamp: 'jvds 10/5/2010 23:27'!
defaultXmlTarget
	"Answer a default xml element name. Use the readSelector if present.
	Otherwise convert the label to a valid element name."

	^MXXmlNamedElementTarget new elementName: self defaultXmlElementName! !

!MADescription methodsFor: '*magritte-xmlbinding' stamp: 'NickAger 2/10/2012 09:38'!
descriptionXmlElementName
	<magritteDescription>
	^ MAStringDescription new 
		accessor: #xmlElementName;
		label: 'XML Element Name';
		priority: 1000;
		yourself! !

!MADescription methodsFor: '*magritte-xmlbinding' stamp: 'NickAger 2/10/2012 09:39'!
descriptionXmlReader
	<magritteDescription>
	^ MASingleOptionDescription new
		accessor: #xmlReader;
		label: 'XML Reader';
		priority: 1300;
		default: self class defaultXmlReader;
		reference: MAClassDescription new;
		options: (Array with: MXXmlReader);
		yourself! !

!MADescription methodsFor: '*magritte-xmlbinding' stamp: 'jvds 10/14/2010 14:24'!
fromXml: aString
	"Answer an object being parsed from <code>aString</code>."

	^ self fromXml: aString reader: self xmlReader! !

!MADescription methodsFor: '*magritte-xmlbinding' stamp: 'jvds 8/31/2010 10:30'!
fromXml: aString reader: aParser
	"Answer an object being parsed from <code>aNode</code> using <code>aParser</code>."

	| node |
	node :=  (XMLDOMParser parseDocumentFrom: aString readStream) root.
	^self fromXmlNode: node reader: aParser! !

!MADescription methodsFor: '*magritte-xmlbinding' stamp: 'jvds 10/6/2010 21:40'!
fromXmlNode: aNode 
	"Answer an object being parsed from <code>aNode</code>."

	^self fromXmlNode: aNode reader: self xmlReader! !

!MADescription methodsFor: '*magritte-xmlbinding' stamp: 'jvds 10/6/2010 21:41'!
fromXmlNode: aNode reader: aXmlReader
	"Answer an object being parsed from <code>aNode</code> using <code>aXmlReader</code>."

	^aXmlReader read: aNode description: self! !

!MADescription methodsFor: '*magritte-xmlbinding' stamp: 'NorbertHartl 2/19/2010 17:53'!
isXmlExportable
	^ self propertyAt: #xmlExportable ifAbsent: [ false ]! !

!MADescription methodsFor: '*magritte-xmlbinding' stamp: 'NorbertHartl 12/8/2011 15:54'!
toXml: anObject
	"Answer a String created from anObject using the receiver as the description."
	
	| xml |
	xml := self toXmlNode: anObject.	
	^String streamContents: [ :stream | xml writeXMLOn: (XMLWriter on: stream)]! !

!MADescription methodsFor: '*magritte-xmlbinding' stamp: 'NorbertHartl 12/21/2010 15:02'!
toXmlDocument: anObject
	"Answer a XML Document created from anObject using the receiver as the description."

	^ (self toXmlNode: anObject) document! !

!MADescription methodsFor: '*magritte-xmlbinding' stamp: 'NorbertHartl 12/21/2010 15:02'!
toXmlNode: anObject
	"Answer a XML Node created from anObject using the receiver as the description."

	^ self toXmlNode: anObject parent: XMLDocument new! !

!MADescription methodsFor: '*magritte-xmlbinding' stamp: 'NorbertHartl 12/21/2010 15:00'!
toXmlNode: anObject parent: aParentNode
	"Answer a XML Node created from anObject using the receiver as the description. If the receivers
	xmlNode is MXSameNode than anObject will be added to aParentNode. Otherwise aParentNode
	will be ignored."

	^ self xmlWriter write: anObject description: self to: aParentNode.
	! !

!MADescription methodsFor: '*magritte-xmlbinding' stamp: 'jvds 10/10/2010 16:00'!
xmlElementName
	^self xmlTarget elementName! !

!MADescription methodsFor: '*magritte-xmlbinding' stamp: 'NickAger 2/3/2012 16:17'!
xmlElementName: aString
	self beXmlExportable.
	self propertyAt: #xmlTarget 
		ifPresent: [ :target | target elementName: aString ] 
		ifAbsent: [ self xmlTarget: (MXXmlNamedElementTarget new elementName: aString) ].
		
	self isKindDefined ifFalse: [
		| describedObject |
		describedObject := self describedObject.
		self kind: (describedObject isNil
			ifTrue: [ MADescription defaultKind ]
			ifFalse: [ describedObject class ]) ]! !

!MADescription methodsFor: '*magritte-xmlbinding' stamp: 'jvds 10/10/2010 15:37'!
xmlReadBlock: aReadBlock writeBlock: aWriteBlock

	| readerAndWriter |
	
	readerAndWriter := MXPluggableXmlAccessor new
		readBlock: aReadBlock;
		writeBlock: aWriteBlock;
		yourself.
		
	self 
		beXmlExportable;
		xmlReader: readerAndWriter;
		xmlWriter: readerAndWriter.! !

!MADescription methodsFor: '*magritte-xmlbinding' stamp: 'jvds 8/31/2010 11:59'!
xmlReader
	^self propertyAt: #xmlReader ifAbsent: [ self class defaultXmlReader ]! !

!MADescription methodsFor: '*magritte-xmlbinding' stamp: 'jvds 8/7/2010 12:19'!
xmlReader: aReader
	self propertyAt: #xmlReader put: aReader! !

!MADescription methodsFor: '*magritte-xmlbinding' stamp: 'jvds 10/5/2010 22:02'!
xmlTarget

	^self propertyAt: #xmlTarget ifAbsentPut: [ self defaultXmlTarget ]! !

!MADescription methodsFor: '*magritte-xmlbinding' stamp: 'jvds 10/5/2010 22:02'!
xmlTarget: aXmlTarget

	^self propertyAt: #xmlTarget put: aXmlTarget! !

!MADescription methodsFor: '*magritte-xmlbinding' stamp: 'jvds 10/10/2010 16:00'!
xmlWriter
	^self propertyAt: #xmlWriter ifAbsent: [ self class defaultXmlWriter ]! !

!MADescription methodsFor: '*magritte-xmlbinding' stamp: 'NorbertHartl 2/21/2010 18:15'!
xmlWriter: aWriter
	self propertyAt: #xmlWriter put: aWriter! !

!MAReferenceDescription methodsFor: '*magritte-xmlbinding' stamp: 'NorbertHartl 7/7/2011 14:35'!
beDescribedByReference

	self propertyAt: #isDescribedByReference put: true! !

!MAReferenceDescription methodsFor: '*magritte-xmlbinding' stamp: 'NorbertHartl 1/25/2011 12:37'!
beInlined
	self xmlTarget beInlined! !

!MAReferenceDescription methodsFor: '*magritte-xmlbinding' stamp: 'NorbertHartl 7/7/2011 14:35'!
isDescribedByReference

	^ self propertyAt: #isDescribedByReference ifAbsent: [ false ] ! !
